/*
 * T_USBCAN.h
 *
 *  Created on: Jul 4, 2020
 *      Author: linkpi
 */

#ifndef T_USBCAN_H_
#define T_USBCAN_H_

#define T_USBCAN_API

//接口卡类型定义
#define VCI_USBCAN1         3
#define VCI_USBCAN2         4
#define VCI_USBCAN2A        4
#define VCI_USBCAN_E_U      20
#define VCI_USBCAN_2E_U     21
#define VCI_USBCANFD_II     41
#define VCI_USBCANFD_I      42

//CAN错误码
#define ERR_CAN_OVERFLOW            0x0001  //CAN控制器内部FIFO溢出
#define ERR_CAN_ERRALARM            0x0002  //CAN控制器错误报警
#define ERR_CAN_PASSIVE             0x0004  //CAN控制器消极错误
#define ERR_CAN_LOSE                0x0008  //CAN控制器仲裁丢失
#define ERR_CAN_BUSERR              0x0010  //CAN控制器总线错误
#define ERR_CAN_BUSOFF              0x0020  //总线关闭错误
#define ERR_CAN_BUFFER_OVERFLOW     0x0040  //CAN控制器内部BUFFER溢出
//通用错误码
#define ERR_DEVICEOPENED            0x0100  //设备已经打开
#define ERR_DEVICEOPEN              0x0200  //打开设备错误
#define ERR_DEVICENOTOPEN           0x0400  //设备没有打开
#define ERR_BUFFEROVERFLOW          0x0800  //缓冲区溢出
#define ERR_DEVICENOTEXIST          0x1000  //此设备不存在
#define ERR_LOADKERNELDLL           0x2000  //装载动态库失败
#define ERR_CMDFAILED               0x4000  //执行命令失败错误码
#define ERR_BUFFERCREATE            0x8000  //内存不足

//函数调用返回状态值
#define	STATUS_OK					1
#define STATUS_ERR					0

//CAN帧类型定义
#define TYPE_CAN_CLASSIC	0
#define TYPE_CAN_FD			1

//USB 一次发送经典CAN帧的数量
#define	MAX_USB_TX_CAN_FRAME_NUMBBER					20

typedef unsigned long       DWORD, ULONG;
typedef int                 INT;
typedef void*				HANDLE;
typedef unsigned char       BYTE;
typedef unsigned short      USHORT;
typedef char				CHAR;
typedef unsigned int        UINT;
typedef unsigned char		UCHAR;
typedef unsigned short      UINT16;
typedef void*				PVOID;
typedef unsigned short      WORD;

//1.TEGCAN系列接口卡信息的数据类型
typedef  struct  _VCI_BOARD_INFO {
	USHORT	hw_Version;
	USHORT	fw_Version;
	USHORT	dr_Version;
	USHORT	in_Version;
	USHORT	irq_Num;
	BYTE	can_Num;
	CHAR	str_Serial_Num[20];
	CHAR	str_hw_Type[40];
	USHORT	Reserved[4];
} VCI_BOARD_INFO, *PVCI_BOARD_INFO;

//2.定义CAN信息帧的数据类型。
typedef  struct  _VCI_CAN_OBJ {
	UINT	ID;
	UINT	TimeStamp;
	BYTE	TimeFlag;
	BYTE	SendType;
	BYTE	RemoteFlag;//是否是远程帧
	BYTE	ExternFlag;//是否是扩展帧
	BYTE	DataLen;
	BYTE    Data[8];
	BYTE    Reserved[3];    //Reserved[0] 第0位表示特殊的空行或者高亮帧
}VCI_CAN_OBJ, *PVCI_CAN_OBJ;

//2+.定义CANFD信息帧的数据类型。
typedef  struct  _VCI_FD_OBJ {
	UINT	ID;
	UINT	TimeStamp;
	BYTE	TimeFlag;
	BYTE	SendType;
	BYTE	RemoteFlag;//是否是远程帧
	BYTE	ExternFlag;//是否是扩展帧
	BYTE	DataLen;
	BYTE    Data[64];
	BYTE    EnBRS;	   //是否使能BRS
	BYTE    CAN_Type;  //CAN类型定义,0为CAN2.O帧，1为CANFD帧
	BYTE    Reserved[1];
}VCI_FD_OBJ, *PVCI_FD_OBJ;

//3.定义CAN控制器状态的数据类型。
typedef struct _VCI_CAN_STATUS {
	UCHAR	ErrInterrupt;
	UCHAR	regMode;
	UCHAR	regStatus;
	UCHAR	regALCapture;
	UCHAR	regECCapture;
	UCHAR	regEWLimit;
	UCHAR	regRECounter;
	UCHAR	regTECounter;
	DWORD	Reserved;
}VCI_CAN_STATUS, *PVCI_CAN_STATUS;

//4.定义错误信息的数据类型。
typedef struct _VCI_ERR_INFO {
	UINT	ErrCode;
	BYTE	Passive_ErrData[3];
	BYTE	ArLost_ErrData;
} VCI_ERR_INFO, *PVCI_ERR_INFO;

//5.定义初始化CAN的数据类型
typedef struct _VCI_INIT_CONFIG {
	DWORD	AccCode;
	DWORD	AccMask;
	DWORD	Reserved;
	UCHAR	Filter;
	UCHAR	Timing0;
	UCHAR	Timing1;
	UCHAR	Mode;

	//设备为USBCANFD-I/II 时使用
	UCHAR    ChannelType;    //CAN通道的类型，0为经典CAN2.0类型，1为CANFD类型
	UCHAR    FD_Standard;    //CANFD的标准，0为ISO标准，1为非ISO标准
	UCHAR	 EnBRS;			 //是否使能BRS，O禁能BRS，1为使能BRS.
	UCHAR    ClockMHz;	     //CAN总线波特率的基准时钟，单位位MHz
	DWORD	 nbtp;			 //仲裁域位时间定义
	DWORD	 dbtp;			 //数据域位时间定义
	UCHAR    EnFilter;		 //是否使能滤波
	UCHAR    WorkMode;       //工作模式，0为正常工作模式，1为只听工作模式
}VCI_INIT_CONFIG, *PVCI_INIT_CONFIG;

//6.定义滤波的数据类型
typedef struct _VCI_FILTER_RECORD {
	DWORD   ExtFrame;   //是否为扩展帧
	DWORD   Start;
	DWORD   End;
}VCI_FILTER_RECORD, *PVCI_FILTER_RECORD;

//7.定时自动发送帧结构
typedef struct _VCI_AUTO_SEND_OBJ {
	BYTE    Enable;     //使能本条报文 0:禁能 1:使能
	BYTE    Index;      //报文编号     最大支持100条报文
	DWORD   Interval;   //定时发送时间 0.1ms为单位
	VCI_CAN_OBJ obj;    //报文
}VCI_AUTO_SEND_OBJ, *PVCI_AUTO_SEND_OBJ;

#ifdef __cplusplus
extern "C" {
#endif

T_USBCAN_API DWORD VCI_OpenDevice(DWORD DeviceType, DWORD DeviceIndex, DWORD Reserved);

T_USBCAN_API DWORD VCI_CloseDevice(DWORD DeviceType, DWORD DeviceInd);
T_USBCAN_API DWORD VCI_InitCAN(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, PVCI_INIT_CONFIG pInitConfig);

T_USBCAN_API DWORD VCI_ReadBoardInfo(DWORD DeviceType, DWORD DeviceInd, PVCI_BOARD_INFO pInfo);
T_USBCAN_API DWORD VCI_ReadErrInfo(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, PVCI_ERR_INFO pErrInfo);
T_USBCAN_API DWORD VCI_ReadCANStatus(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, PVCI_CAN_STATUS pCANStatus);

T_USBCAN_API DWORD VCI_GetReference(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, DWORD RefType, PVOID pData);
T_USBCAN_API DWORD VCI_SetReference(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, DWORD RefType, PVOID pData);

T_USBCAN_API ULONG VCI_GetReceiveNum(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd);
T_USBCAN_API DWORD VCI_ClearBuffer(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd);

T_USBCAN_API DWORD VCI_StartCAN(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd);
T_USBCAN_API DWORD VCI_ResetCAN(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd);

T_USBCAN_API ULONG VCI_Transmit(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, PVCI_CAN_OBJ pSend, ULONG Len);
T_USBCAN_API ULONG VCI_Receive(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, PVCI_CAN_OBJ pReceive, ULONG Len, INT WaitTime);

T_USBCAN_API ULONG VCI_TransmitFD(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, PVCI_FD_OBJ pSend, ULONG Len);
T_USBCAN_API ULONG VCI_ReceiveFD(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, PVCI_FD_OBJ pReceive, ULONG Len, INT WaitTime);

T_USBCAN_API DWORD VCI_SetSendInterval(DWORD DeviceType, DWORD DeviceInd, DWORD CANInd, DWORD Interval);

//__inline void format_canobj_to_canmsg(PVCI_CAN_OBJ pSend, DWORD CANInd, ULONG Len, UCHAR *buff_out);


#ifdef __cplusplus
}
#endif


#endif /* T_USBCAN_H_ */
