/*
 ============================================================================
 Name        : demo_usbcan.c
 Author      : kingreat
 Version     :
 Copyright   : Beijing iTegva Technology Co., Ltd.
 Description : Hello World in C, Ansi-style
 ============================================================================
 */

/* ------------------------------ includes ----------------------------------*/
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <pthread.h>
#include <signal.h>
#include <unistd.h>


#include "T_USBCAN.h"



/* ----------------------- Defines ------------------------------------------*/
#define DEVICE_IND 0
#define MAX_RECV_MSG_NUMBER	100

#define STATE_TASK_IS_RUNNING	0
#define STATE_TASK_DO_EXIT	    1
#define STATE_TASK_IS_END		2

pthread_t tid_send;
pthread_t tid_recv;

uint8_t state_recv_task = STATE_TASK_IS_RUNNING;
uint8_t state_send_task = STATE_TASK_IS_RUNNING;


/* ----------------------- Function prototype -------------------------------*/
void ctrl_c_handle(int signo)
{
    int retry_timers;

    signal (SIGINT, NULL);

    printf ("oops! enter ctrl_c_handle!!!\n");

    /** Safely Exit the send thread */
    retry_timers = 0;
    state_send_task = STATE_TASK_DO_EXIT;
    while(state_send_task != STATE_TASK_IS_END) {
        sleep(1);
        if(++retry_timers == 5) break;
    }

    /** Safely Exit the receive thread */
    retry_timers = 0;
    state_recv_task = STATE_TASK_DO_EXIT;
    while(state_recv_task != STATE_TASK_IS_END) {
        sleep(1);
        if(++retry_timers == 5) break;
    }

    VCI_CloseDevice(VCI_USBCAN2, DEVICE_IND);

    printf ("oops! exit ctrl_c_handle!!!\n");
    exit (0);
}

void *task_send_canmsg (void *arg)
{
    VCI_CAN_OBJ send_msg[3];
    uint8_t data[8] = {1,2,3,4,5,6,7,8};

    memset (send_msg, 0 ,sizeof(VCI_CAN_OBJ) * 3);

    uint32_t i = 0;

    while(1) {

        if(state_send_task == STATE_TASK_DO_EXIT) {
            state_send_task = STATE_TASK_IS_END;
            return NULL;
        }

        send_msg[0].ID = i++;
        memcpy(send_msg[0].Data, data, 8);
        send_msg[0].DataLen = sizeof(data);
        send_msg[0].ExternFlag = 0;
        send_msg[0].RemoteFlag = 0;
        send_msg[0].SendType   = 0;
        send_msg[0].TimeFlag   = 1;
        send_msg[0].TimeStamp  = 0x12345678;

        data[7]++;
        send_msg[1].ID = i++;
        memcpy(send_msg[1].Data, data, 8);
        send_msg[1].DataLen    = sizeof(data);
        send_msg[1].ExternFlag = 0;
        send_msg[1].RemoteFlag = 0;
        send_msg[1].SendType   = 0;
        send_msg[1].TimeFlag   = 1;
        send_msg[1].TimeStamp  = 0x12345678;

        data[7]++;
        send_msg[2].ID = i++;
        memcpy(send_msg[2].Data, data, 8);
        send_msg[2].DataLen    = sizeof(data);
        send_msg[2].ExternFlag = 0;
        send_msg[2].RemoteFlag = 0;
        send_msg[2].SendType   = 0;
        send_msg[2].TimeFlag   = 1;
        send_msg[2].TimeStamp  = 0x12345678;

        int ret = VCI_Transmit (VCI_USBCAN2, DEVICE_IND, 0, send_msg, 3);
        if (ret == 0) {
            break;
        }

#if 1
        for (int i = 0; i < 3; i++) {

            printf("TX: [ID:%08x(%c %c) Data:", send_msg[i].ID,
                                                send_msg[i].ExternFlag ? 'E':'S',
                                                send_msg[i].RemoteFlag ? 'R':'D');

            for (int j=0; j< send_msg[i].DataLen; j++) {
                printf(" %02x", send_msg[i].Data[j]);
            }
            printf("]\n");

        }
#endif


        if (data[7] >= 255) data[7] = 0;


        usleep (100 * 1000);
    }


    printf("Exit the send can message task\n");

    return NULL;
}

void *task_recv_canmsg (void *arg)
{
    VCI_CAN_OBJ recv_msg[MAX_RECV_MSG_NUMBER];

    uint32_t ch = 0;

    uint32_t index = 0;

    while (1) {

        if(state_recv_task == STATE_TASK_DO_EXIT) {
            state_recv_task = STATE_TASK_IS_END;
            return NULL;
        }


        int num = VCI_GetReceiveNum(VCI_USBCAN2, DEVICE_IND, ch);

        num = (num > MAX_RECV_MSG_NUMBER) ? MAX_RECV_MSG_NUMBER : num;

        if (num > 0) {
            VCI_Receive(VCI_USBCAN2, DEVICE_IND, ch, recv_msg, num, 0);

            for (int i = 0; i < num; i++) {

                printf("RX: [ID:%08x(%c %c) Data:", recv_msg[i].ID,
                                                recv_msg[i].ExternFlag ? 'E':'S',
                                                recv_msg[i].RemoteFlag ? 'R':'D');

                for (int j=0; j< recv_msg[i].DataLen; j++) {
                    printf(" %02x", recv_msg[i].Data[j]);
                }
                printf("] %d\n", ++index);

            }
        }

        if (++ch >= 2) ch = 0;
    }
}


int main(void) {

    if (VCI_OpenDevice(VCI_USBCAN2, DEVICE_IND, 0) == 0) {
        goto exit;
    }


    VCI_INIT_CONFIG init_config;

    memset(&init_config, 0, sizeof(VCI_INIT_CONFIG));
    init_config.AccCode = 0x00000000;
    init_config.AccMask = 0xFFFFFFFF;
    init_config.Filter  = 0;
    init_config.Timing0 = 0x00;	// 1000kbps
    init_config.Timing1 = 0x14;
    init_config.Mode    = 0; 	// Normal Workmode

    /** Initialize channel 0 */
    if (VCI_InitCAN(VCI_USBCAN2, DEVICE_IND, 0, &init_config) == 0) {
        goto exit;
    }

    /** Initialize channel 1 */
    if (VCI_InitCAN(VCI_USBCAN2, DEVICE_IND, 1, &init_config) == 0) {
        goto exit;
    }

    /** Get board infomation */
    VCI_BOARD_INFO board_info;
    if (VCI_ReadBoardInfo(VCI_USBCAN2, 0, &board_info) == 0) {
        goto exit;
    }
    printf("===================================\n");
    printf("=== Hardware Version: %04x\n", board_info.hw_Version);
    printf("=== Firmware Version: %04x\n", board_info.hw_Version);
    printf("=== Product serialno: %s\n",   board_info.str_Serial_Num);
    printf("=== Hardware    type: %s\n",   board_info.str_hw_Type);
    printf("===================================\n");

    /** Start the CAN channel */
    if (VCI_StartCAN (VCI_USBCAN2, DEVICE_IND, 0) == 0) {
        goto exit;
    }

    if (VCI_StartCAN (VCI_USBCAN2, DEVICE_IND, 1) == 0) {
        goto exit;
    }

    /** Create receive and send can message task */
    int32_t err = -1;


    if((err = pthread_create(&tid_send, NULL, task_send_canmsg, NULL))) {
        printf ("-E- create send can message thread error: %s\n", strerror(err));
        goto exit;
    }
#if 1
    if ((err = pthread_create(&tid_recv, NULL, task_recv_canmsg, NULL))) {
        printf ("-E- create recv can message thread error: %s\n", strerror(err));
        goto exit;
    }
#endif

    signal (SIGINT, ctrl_c_handle);

    while (1) {
        usleep(10 * 1000);
    }

    return EXIT_SUCCESS;

exit:
    VCI_CloseDevice(VCI_USBCAN2, DEVICE_IND);

    return EXIT_FAILURE;
}
